; Copyright (c) 2011, Vojtech Salajka
; All rights reserved.
; 
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are met:
;     * Redistributions of source code must retain the above copyright
;       notice, this list of conditions and the following disclaimer.
;     * Redistributions in binary form must reproduce the above copyright
;       notice, this list of conditions and the following disclaimer in the
;       documentation and/or other materials provided with the distribution.
;     * Neither the name of the Franticware nor the
;       names of its contributors may be used to endorse or promote products
;       derived from this software without specific prior written permission.
; 
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
; DISCLAIMED. IN NO EVENT SHALL Vojtech Salajka BE LIABLE FOR ANY
; DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
; (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
; ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
; (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

; Include derivative-specific definitions
            INCLUDE 'derivative.inc'
            XREF MCU_init

; export symbols
            XDEF _Startup, main
            ; we export both '_Startup' and 'main' as symbols. Either can
            ; be referenced in the linker .prm file or from C/C++ later on

            XREF __SEG_END_SSTACK   ; symbol defined by the linker for the end of the stack                           

    INCLUDE 'sleepm.inc'

Nsync: equ 3 ; neutral sync
Hsync: equ 1 ; inverse hsync
Vsync: equ 2 ; inverse vsync
Red:   equ 4 ; red component
Green: equ 8 ; green component
Blue:  equ 16; blue component 
White: equ Nsync+Red+Green+Blue
Black: equ Nsync            

; variable/data section
MY_ZEROPAGE: SECTION  SHORT         ; Insert here your data definition

linec:    
    ds.b 20
linec2:
    ds.b 10
linec3:
    ds.b 16        
linerot:
    ds.b 1    
boola:
    ds.b 1  

DEFAULT_ROM SECTION
     
text0:
    dc.b White, White, White, White, White, White, White, White, White, White, White, White, White, White, White, White
text1:
    dc.b White, Black, Black, White, White, Black, Black, White, White, Black, Black, White, White, Black, Black, White    
text2:
    dc.b White, Black, Black, Black, Black, Black, Black, White, White, Black, Black, White, White, Black, Black, White    
text3:
    dc.b White, Black, Black, Black, Black, Black, Black, White, White, Black, Black, White, White, White, White, White    


; code section
MyCode:     SECTION
main:
_Startup:
    LDHX   #__SEG_END_SSTACK ; initialize the stack pointer
    TXS
    ; Call generated Device Initialization function
    JSR    MCU_init           
    
    lda #Nsync+Red ; red
    sta linec
    sta linec+1     
    lda #Nsync+Red+Green ; yellow
    sta linec+2
    sta linec+3           
    lda #Nsync+Green ; green
    sta linec+4
    sta linec+5          
    lda #Nsync+Green+Blue ; azure
    sta linec+6
    sta linec+7      
    lda #Nsync+Blue ; blue
    sta linec+8
    sta linec+9
    lda #Nsync+Blue+Red ; purple
    sta linec+10
    sta linec+11
    lda #Nsync+Blue ; blue
    sta linec+12
    sta linec+13
    lda #Nsync+Green+Blue ; azure
    sta linec+14
    sta linec+15
    lda #Nsync+Green ; green
    sta linec+16
    sta linec+17    
    lda #Nsync+Red+Green ; yellow
    sta linec+18
    sta linec+19
    lda #Nsync
    sta linec+20
    sta linec+21
    sta linec+22
    sta linec+23
    sta linec+24
    sta linec+25
    sta linec+26
    sta linec+27
    sta linec+28
    sta linec+29    
    
    lda #0
    sta boola
                

mainLoop:
    ;mov #0, PTAD           ; generate 440 hz on pta0
    ;sleep27268
    ;mov #1, PTAD
    ;sleep27264
    ;jmp mainLoop

    ; --- line0
    lda #Nsync              ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 44+640+20=704
    sleep699                ; OK
    lda #Nsync-Hsync-Vsync  ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 96
    sleep91                 ; OK
    
    ; --- line1
    lda #Nsync-Vsync        ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 44+640+20=704
    sleep699                ; OK
    lda #Nsync-Hsync
    sta PTAD                
    ; --- 96
    sleep91                 ; OK
    
    ; --- line2
    lda #Nsync
    sta PTAD
    ; --- 44+640+20=704
    
    lda boola               ; 3 cyc
    tsta                    ; 1 cyc
    bne rotate              ; 3 cyc
    jmp norotate            ; 4 cyc
rotate:
    lda #1                  ; 2 cyc ; set this to 0 for half scrolling speed
    sta boola               ; 3 cyc
    lda linec               ; 3 cyc
    sta linerot             ; 3 cyc
    FOR i=0 TO 18           ; 19 times
    lda linec+i+1           ; 3 cyc
    sta linec+i             ; 3 cyc
    ENDFOR    
    lda linerot             ; 3 cyc
    sta linec+19            ; 3 cyc
    FOR i=0 TO 9
    lda linec+i
    sta linec2+i
    ENDFOR
    FOR i=0 TO 15           ; 16 times
    lda linec+i
    sta linec3+15-i
    ENDFOR
    sleep402                ; OK
    bra endrotate           ; 3 cyc
norotate:
    lda #1                  ; 2 cyc
    sta boola               ; 3 cyc
    sleep683                ; OK
endrotate:
    lda #Nsync-Hsync        ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 96
    sleep84                 ; OK
    
    ; --- 30 intro lines
    lda #29                 ; 2 cyc
    bra loop0b              ; 3 cyc
loop0a:
    sleep77                 ; OK
loop0b:
    psha                    ; 2 cyc
    lda #Nsync              ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 704
    sleep699                ; OK
    lda #Nsync-Hsync
    sta PTAD
    ; --- 96
    pula                    ; 3 cyc
    deca                    ; 1 cyc
    tsta                    ; 1 cyc
    beq loop0c              ; 3 cyc
    jmp loop0a              ; 4 cyc
loop0c:

    sleep67                 ; OK

    ; 21 cycles to value change
    lda #80                ; 2 cyc
    ldhx #linec3         ; 3 cyc
    jsr drawlines16
    ; 14 cycles from value change
    
    sleep61                 ; OK
    
    ; 21 cycles to value change
    lda #40 ; 2 cyc
    ldhx #text0 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #80 ; 2 cyc
    ldhx #text1 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #60 ; 2 cyc
    ldhx #text2 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #20 ; 2 cyc
    ldhx #text3 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #80 ; 2 cyc
    ldhx #text1 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #40 ; 2 cyc
    ldhx #text0 ; 3 cyc
    jsr drawlines16
    
    sleep61                 ; OK
    
    lda #80
    ldhx #linec
    jsr drawlines16
    ; 14 cycles from value change
    
    sleep70                 ; OK

    ; --- 13 outro lines
    lda #13                 ; 2 cyc
    bra loop1b              ; 3 cyc
loop1a:
    sleep77                 ; OK
loop1b:
    psha                    ; 2 cyc
    lda #Nsync
    sta PTAD
    ; --- 704
    sleep699
    lda #Nsync-Hsync
    sta PTAD
    ; --- 96
    pula                    ; 3 cyc
    deca                    ; 1 cyc
    tsta                    ; 1 cyc
    beq loop1c              ; 3 cyc
    jmp loop1a              ; 4 cyc
loop1c:    
    
    sleep79
    jmp mainLoop            ; 4 cyc
    
        
; 16 cycles to value change    
drawlines16: ; params: a = line count, hx = line colors  ; 
    bra loopxb              ; 3 cyc
loopxa:
    sleep77                 ; OK
loopxb:
    psha                    ; 2 cyc
    lda #Nsync              ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 44
    
    sleep37                 ; OK

    FOR i=0 TO 15           ; 7 cyc
    nop   
    lda i, X                ; 3 cyc    
    sta PTAD                ; 3 cyc
    ; --- 40                ; OK
    sleep33
    ENDFOR
    
    nop
    nop
    lda #Nsync              ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 20
    sleep15                 ; OK
    lda #Nsync-Hsync        ; 2 cyc
    sta PTAD                ; 3 cyc
    ; --- 96
    pula                    ; 3 cyc
    deca                    ; 1 cyc
    tsta                    ; 1 cyc
    beq loopxc              ; 3 cyc
    jmp loopxa              ; 4 cyc
loopxc:
    rts ; 6 cyc   
; 14 cycles from value change    

    INCLUDE 'sleepr.inc'